@@ -13,6 +13,10 @@ import com.unciv.logic.map.mapunit.MapUnit
 import com.unciv.logic.map.tile.Tile
 import com.unciv.logic.multiplayer.isUsersTurn
 import com.unciv.models.ruleset.Building
+import com.unciv.models.ruleset.IConstruction
+import com.unciv.models.ruleset.INonPerpetualConstruction
+import com.unciv.models.ruleset.PerpetualConstruction
+import com.unciv.models.ruleset.RejectionReasonType
 import com.unciv.models.ruleset.Ruleset
 import com.unciv.models.ruleset.unique.LocalUniqueCache
 import com.unciv.models.ruleset.unique.StateForConditionals
@@ -127,7 +131,7 @@ class CityConstructions : IsPartOfGameInfoSerialization {
 
     fun getCityProductionTextForCityButton(): String {
         val currentConstructionSnapshot = currentConstructionFromQueue // See below
-        var result = currentConstructionSnapshot.tr()
+        var result = currentConstructionSnapshot.tr(true)
         if (currentConstructionSnapshot.isNotEmpty()) {
             val construction = PerpetualConstruction.perpetualConstructionsMap[currentConstructionSnapshot]
             result += construction?.getProductionTooltip(city)
@@ -349,7 +353,19 @@ class CityConstructions : IsPartOfGameInfoSerialization {
         constructionQueue.clear()
 
         for (constructionName in queueSnapshot) {
-            if (getConstruction(constructionName).isBuildable(this))
+            val construction = getConstruction(constructionName)
+            // First construction will be built next turn, we need to make sure it has the correct resources
+            if (constructionQueue.isEmpty() && getWorkDone(constructionName) == 0) {
+                val costUniques = construction.getMatchingUniquesNotConflicting(UniqueType.CostsResources)
+                val civResources = city.civ.getCivResourcesByName()
+
+                if (costUniques.any {
+                            val resourceName = it.params[1]
+                            civResources[resourceName] == null
+                                    || it.params[0].toInt() > civResources[resourceName]!! })
+                    continue // Removes this construction from the queue
+            }
+            if (construction.isBuildable(this))
                 constructionQueue.add(constructionName)
         }
     }
@@ -392,6 +408,14 @@ class CityConstructions : IsPartOfGameInfoSerialization {
     }
 
     private fun constructionBegun(construction: IConstruction) {
+        val costUniques = construction.getMatchingUniquesNotConflicting(UniqueType.CostsResources)
+
+        for (unique in costUniques){
+            val amount = unique.params[0].toInt()
+            val resourceName = unique.params[1]
+            city.civ.resourceStockpiles.add(resourceName, -amount)
+        }
+
         if (construction !is Building) return
         if (!construction.hasUnique(UniqueType.TriggersAlertOnStart)) return
         val buildingIcon = "BuildingIcons/${construction.name}"
