@@ -16,12 +16,14 @@ import com.unciv.models.ruleset.unique.UniqueType
 import java.lang.Integer.max
 import kotlin.math.abs
 
-/** An Unciv map with all properties as produced by the [map editor][com.unciv.ui.mapeditor.MapEditorScreen]
+/** An Unciv map with all properties as produced by the [map editor][com.unciv.ui.screens.mapeditorscreen.MapEditorScreen]
  * or [MapGenerator][com.unciv.logic.map.mapgenerator.MapGenerator]; or as part of a running [game][GameInfo].
  *
  * Note: Will be Serialized -> Take special care with lateinit and lazy!
+ *
+ * @param initialCapacity Passed to constructor of [tileList]
  */
-class TileMap : IsPartOfGameInfoSerialization {
+class TileMap(initialCapacity: Int = 10) : IsPartOfGameInfoSerialization {
     companion object {
         /** Legacy way to store starting locations - now this is used only in [translateStartingLocationsFromMap] */
         const val startingLocationPrefix = "StartingLocation "
@@ -38,7 +40,7 @@ class TileMap : IsPartOfGameInfoSerialization {
 
     var mapParameters = MapParameters()
 
-    var tileList = ArrayList<Tile>()
+    var tileList = ArrayList<Tile>(initialCapacity)
 
     /** Structure geared for simple serialization by Gdx.Json (which is a little blind to kotlin collections, especially HashSet)
      * @param position [Vector2] of the location
@@ -93,11 +95,9 @@ class TileMap : IsPartOfGameInfoSerialization {
     //endregion
     //region Constructors
 
-    /** for json parsing, we need to have a default constructor */
-    constructor()
-
     /** creates a hexagonal map of given radius (filled with grassland) */
-    constructor(radius: Int, ruleset: Ruleset, worldWrap: Boolean = false) {
+    constructor(radius: Int, ruleset: Ruleset, worldWrap: Boolean = false)
+            : this (HexMath.getNumberOfTilesInHexagon(radius)) {
         startingLocations.clear()
         val firstAvailableLandTerrain = MapLandmassGenerator.getInitializationTerrain(ruleset, TerrainType.Land)
         for (vector in HexMath.getVectorsInDistance(Vector2.Zero, radius, worldWrap))
@@ -106,7 +106,8 @@ class TileMap : IsPartOfGameInfoSerialization {
     }
 
     /** creates a rectangular map of given width and height (filled with grassland) */
-    constructor(width: Int, height: Int, ruleset: Ruleset, worldWrap: Boolean = false) {
+    constructor(width: Int, height: Int, ruleset: Ruleset, worldWrap: Boolean = false)
+            : this(width * height) {
         startingLocations.clear()
         val firstAvailableLandTerrain = MapLandmassGenerator.getInitializationTerrain(ruleset, TerrainType.Land)
 
@@ -130,13 +131,16 @@ class TileMap : IsPartOfGameInfoSerialization {
 
     /** @return a deep-copy clone of the serializable fields, no transients initialized */
     fun clone(): TileMap {
-        val toReturn = TileMap()
-        toReturn.tileList.addAll(tileList.map { it.clone() })
+        val toReturn = TileMap(tileList.size)
+        toReturn.tileList.addAll(tileList.asSequence().map { it.clone() })
         toReturn.mapParameters = mapParameters
         toReturn.ruleset = ruleset
+
+        // Note during normal play this is empty. Supported for MapEditorScreen.getMapCloneForSave.
         toReturn.startingLocations.clear()
         toReturn.startingLocations.ensureCapacity(startingLocations.size)
         toReturn.startingLocations.addAll(startingLocations)
+
         return toReturn
     }
 
@@ -205,29 +209,17 @@ class TileMap : IsPartOfGameInfoSerialization {
                 }.filterNotNull()
 
     /** @return all tiles within [rectangle], respecting world edges and wrap.
-     *  If using row/column coordinates the rectangle will be "straight" ie parallel with rectangular map edges. */
-    fun getTilesInRectangle(rectangle: Rectangle, rowsAndColumns: Boolean = false): Sequence<Tile> =
-            if (rectangle.width <= 0 || rectangle.height <= 0) {
-                val tile = getIfTileExistsOrNull(rectangle.x.toInt(), rectangle.y.toInt())
-                if (tile == null) sequenceOf()
-                else sequenceOf(tile)
+     *  The rectangle will be "straight" ie parallel with rectangular map edges. */
+    fun getTilesInRectangle(rectangle: Rectangle) = sequence {
+            val x = rectangle.x.toInt()
+            val y = rectangle.y.toInt()
+            for (worldColumnNumber in x until x + rectangle.width.toInt()) {
+                for (worldRowNumber in y until y + rectangle.height.toInt()) {
+                    val hexCoords = HexMath.getTileCoordsFromColumnRow(worldColumnNumber, worldRowNumber)
+                    yield(getIfTileExistsOrNull(hexCoords.x.toInt(), hexCoords.y.toInt()))
+                }
             }
-            else
-                sequence {
-                    for (rectColumnNumber in 0 until rectangle.width.toInt()) {
-                        for (rectRowNumber in 0 until rectangle.height.toInt()) {
-                            val worldColumnNumber = rectangle.x.toInt() + rectColumnNumber
-                            val worldRowNumber = rectangle.y.toInt() + rectRowNumber
-
-                            if (rowsAndColumns) {
-                                val hexCoords = HexMath.getTileCoordsFromColumnRow(worldColumnNumber, worldRowNumber)
-                                yield(getIfTileExistsOrNull(hexCoords.x.toInt(), hexCoords.y.toInt()))
-                            }
-                            else
-                                yield(getIfTileExistsOrNull(worldColumnNumber, worldRowNumber))
-                        }
-                    }
-                }.filterNotNull()
+        }.filterNotNull()
 
     /** @return tile at hex coordinates ([x],[y]) or null if they are outside the map. Respects map edges and world wrap. */
     fun getIfTileExistsOrNull(x: Int, y: Int): Tile? {
@@ -393,9 +385,9 @@ class TileMap : IsPartOfGameInfoSerialization {
             rulesetIncompatibilities.addAll(set)
 
         // All the rest is to find missing nations
-        try { // This can fail if the map contains a resource that isn't in the ruleset, in TileInfo.tileResource
+        try { // This can fail if the map contains a resource that isn't in the ruleset, in Tile.tileResource
             setTransients(ruleset)
-        } catch (ex: Exception) {
+        } catch (_: Exception) {
             return rulesetIncompatibilities
         }
         setStartingLocationsTransients()
@@ -423,7 +415,7 @@ class TileMap : IsPartOfGameInfoSerialization {
      */
     fun setTransients(ruleset: Ruleset? = null, setUnitCivTransients: Boolean = true) {
         if (ruleset != null) this.ruleset = ruleset
-        if (this.ruleset == null) throw(IllegalStateException("TileMap.setTransients called without ruleset"))
+        check(this.ruleset != null) { "TileMap.setTransients called without ruleset" }
 
         if (tileMatrix.isEmpty()) {
             val topY = tileList.asSequence().map { it.position.y.toInt() }.maxOrNull()!!
@@ -442,18 +434,19 @@ class TileMap : IsPartOfGameInfoSerialization {
         } else {
             // Yes the map generator calls this repeatedly, and we don't want to end up with an oversized tileMatrix
             // rightX is -leftX or -leftX + 1 or -leftX + 2
-            if (tileMatrix.size !in (1 - 2 * leftX)..(3 - 2 * leftX))
-                throw(IllegalStateException("TileMap.setTransients called on existing tileMatrix of different size"))
+            check(tileMatrix.size in (1 - 2 * leftX)..(3 - 2 * leftX)) {
+                "TileMap.setTransients called on existing tileMatrix of different size"
+            }
         }
 
         for (tileInfo in values) {
             tileMatrix[tileInfo.position.x.toInt() - leftX][tileInfo.position.y.toInt() - bottomY] = tileInfo
         }
         for (tileInfo in values) {
-            // Do ***NOT*** call TileInfo.setTerrainTransients before the tileMatrix is complete -
+            // Do ***NOT*** call Tile.setTerrainTransients before the tileMatrix is complete -
             // setting transients might trigger the neighbors lazy (e.g. thanks to convertHillToTerrainFeature).
             // When that lazy runs, some directions might be omitted because getIfTileExistsOrNull
-            // looks at tileMatrix. Thus filling TileInfos into tileMatrix and setting their
+            // looks at tileMatrix. Thus filling Tiles into tileMatrix and setting their
             // transients in the same loop will leave incomplete cached `neighbors`.
             tileInfo.tileMap = this
             tileInfo.ruleset = this.ruleset!!
@@ -462,8 +455,8 @@ class TileMap : IsPartOfGameInfoSerialization {
         }
     }
 
-    /** Initialize based on TileInfo which Civ has neutral tile roads
-     */
+    /** Initialize Civilization.neutralRoads based on Tile.roadOwner
+     *  - which Civ owns roads on which neutral tiles */
     fun setNeutralTransients() {
         for (tileInfo in values) {
             tileInfo.setOwnerTransients()
@@ -550,7 +543,7 @@ class TileMap : IsPartOfGameInfoSerialization {
         // And update civ stats, since the new unit changes both unit upkeep and resource consumption
         civInfo.updateStatsForNextTurn()
 
-        if (unit.baseUnit.getResourceRequirements().isNotEmpty())
+        if (unit.baseUnit.getResourceRequirementsPerTurn().isNotEmpty())
             civInfo.cache.updateCivResources()
 
         return unit
