@@ -2,7 +2,7 @@
 
 import com.badlogic.gdx.math.Vector2
 import com.unciv.Constants
-import com.unciv.UncivGame
+import com.unciv.GUI
 import com.unciv.logic.IsPartOfGameInfoSerialization
 import com.unciv.logic.city.City
 import com.unciv.logic.civilization.Civilization
@@ -21,6 +21,9 @@ import com.unciv.models.ruleset.unique.StateForConditionals
 import com.unciv.models.ruleset.unique.Unique
 import com.unciv.models.ruleset.unique.UniqueMap
 import com.unciv.models.ruleset.unique.UniqueType
+import com.unciv.ui.components.extensions.withItem
+import com.unciv.ui.components.extensions.withoutItem
+import com.unciv.utils.DebugUtils
 import kotlin.math.abs
 import kotlin.math.min
 import kotlin.random.Random
@@ -52,11 +55,9 @@ open class Tile : IsPartOfGameInfoSerialization {
                 getRoadOwner()!!.neutralRoads.remove(this.position)
             }
             roadOwner = city.civ.civName // only when taking control, otherwise last owner
-        } else {
-            if (roadStatus != RoadStatus.None && owningCity != null) {
-                // previous tile owner still owns road, add to tracker
-                owningCity!!.civ.neutralRoads.add(this.position)
-            }
+        } else if (roadStatus != RoadStatus.None && owningCity != null) {
+            // previous tile owner still owns road, add to tracker
+            owningCity!!.civ.neutralRoads.add(this.position)
         }
         owningCity = city
         isCityCenterInternal = getCity()?.location == position
@@ -84,6 +85,7 @@ open class Tile : IsPartOfGameInfoSerialization {
     var terrainFeatures: List<String> = listOf()
         private set
 
+    /** Should be immutable - never be altered in-place, instead replaced */
     var exploredBy = HashSet<String>()
 
     @Transient
@@ -95,6 +97,10 @@ open class Tile : IsPartOfGameInfoSerialization {
     var allTerrains: Sequence<Terrain> = sequenceOf()
         private set
 
+    @Transient
+    lateinit var lastTerrain: Terrain
+        private set
+
     @Transient
     var terrainUniqueMap = UniqueMap()
         private set
@@ -129,6 +135,8 @@ open class Tile : IsPartOfGameInfoSerialization {
     var hasBottomRiver = false
     var hasBottomLeftRiver = false
 
+    var history: TileHistory = TileHistory()
+
     private var continent = -1
 
     val latitude: Float
@@ -167,7 +175,8 @@ open class Tile : IsPartOfGameInfoSerialization {
         toReturn.hasBottomRightRiver = hasBottomRightRiver
         toReturn.hasBottomRiver = hasBottomRiver
         toReturn.continent = continent
-        toReturn.exploredBy.addAll(exploredBy)
+        toReturn.exploredBy = exploredBy
+        toReturn.history = history.clone()
         return toReturn
     }
 
@@ -177,15 +186,6 @@ open class Tile : IsPartOfGameInfoSerialization {
         return getTileImprovement()?.isGreatImprovement() == true
     }
 
-    fun containsUnpillagedGreatImprovement(): Boolean {
-        return getUnpillagedTileImprovement()?.isGreatImprovement() == true
-    }
-
-    fun containsUnfinishedGreatImprovement(): Boolean {
-        if (improvementInProgress == null) return false
-        return ruleset.tileImprovements[improvementInProgress!!]!!.isGreatImprovement()
-    }
-
     /** Returns military, civilian and air units in tile */
     fun getUnits() = sequence {
         if (militaryUnit != null) yield(militaryUnit!!)
@@ -210,13 +210,6 @@ open class Tile : IsPartOfGameInfoSerialization {
 
     fun getCity(): City? = owningCity
 
-    fun getLastTerrain(): Terrain = when {
-        terrainFeatures.isNotEmpty() -> ruleset.terrains[terrainFeatures.last()]
-                ?: getBaseTerrain()  // defense against rare edge cases involving baseTerrain Hill deprecation
-        naturalWonder != null -> getNaturalWonder()
-        else -> getBaseTerrain()
-    }
-
     @Transient
     private var tileResourceCache: TileResource? = null
     val tileResource: TileResource
@@ -234,30 +227,38 @@ open class Tile : IsPartOfGameInfoSerialization {
             else ruleset.terrains[naturalWonder!!]!!
 
     fun isVisible(player: Civilization): Boolean {
-        if (UncivGame.Current.viewEntireMapForDebug)
+        if (DebugUtils.VISIBLE_MAP)
             return true
         return player.viewableTiles.contains(this)
     }
 
     fun isExplored(player: Civilization): Boolean {
-        if (UncivGame.Current.viewEntireMapForDebug || player.isSpectator())
+        if (DebugUtils.VISIBLE_MAP || player.civName == Constants.spectator)
             return true
-        return exploredBy.contains(player.civName) || player.exploredTiles.contains(position)
+        return exploredBy.contains(player.civName)
     }
 
-    fun setExplored(player: Civilization, isExplored: Boolean) {
+    fun setExplored(player: Civilization, isExplored: Boolean, explorerPosition: Vector2? = null) {
         if (isExplored) {
-            exploredBy.add(player.civName)
-            player.exploredTiles.add(position)
+            // Disable the undo button if a new tile has been explored
+            if (!exploredBy.contains(player.civName)) {
+                if (GUI.isWorldLoaded()) {
+                    val worldScreen = GUI.getWorldScreen()
+                    worldScreen.preActionGameInfo = worldScreen.gameInfo
+                }
+                exploredBy = exploredBy.withItem(player.civName)
+            }
+
+            if (player.playerType == PlayerType.Human)
+                player.exploredRegion.checkTilePosition(position, explorerPosition)
         } else {
-            exploredBy.remove(player.civName)
-            player.exploredTiles.remove(position)
+            exploredBy = exploredBy.withoutItem(player.civName)
         }
     }
 
     fun isCityCenter(): Boolean = isCityCenterInternal
     fun isNaturalWonder(): Boolean = naturalWonder != null
-    fun isImpassible() = getLastTerrain().impassable
+    fun isImpassible() = lastTerrain.impassable
 
     fun getTileImprovement(): TileImprovement? = if (improvement == null) null else ruleset.tileImprovements[improvement!!]
     fun getUnpillagedTileImprovement(): TileImprovement? = if (getUnpillagedImprovement() == null) null else ruleset.tileImprovements[improvement!!]
@@ -489,6 +490,20 @@ open class Tile : IsPartOfGameInfoSerialization {
         return fertility
     }
 
+    fun providesResources(civInfo: Civilization): Boolean {
+        if (!hasViewableResource(civInfo)) return false
+        if (isCityCenter()) return true
+        val improvement = getUnpillagedTileImprovement()
+        if (improvement != null && improvement.name in tileResource.getImprovements()
+                && (improvement.techRequired==null || civInfo.tech.isResearched(improvement.techRequired!!))) return true
+        // TODO: Generic-ify to unique
+        if (tileResource.resourceType==ResourceType.Strategic
+                && improvement!=null
+                && improvement.isGreatImprovement())
+            return true
+        return false
+    }
+
     // This should be the only adjacency function
     fun isAdjacentTo(terrainFilter:String): Boolean {
         // Rivers are odd, as they aren't technically part of any specific tile but still count towards adjacency
@@ -524,9 +539,11 @@ open class Tile : IsPartOfGameInfoSerialization {
             "Natural Wonder" -> naturalWonder != null
             "Featureless" -> terrainFeatures.isEmpty()
             Constants.freshWaterFilter -> isAdjacentTo(Constants.freshWater)
+
+            in terrainFeatures -> true
             else -> {
-                if (terrainFeatures.contains(filter)) return true
                 if (terrainUniqueMap.getUniques(filter).any()) return true
+                if (getOwner()?.nation?.matchesFilter(filter) == true) return true
 
                 // Resource type check is last - cannot succeed if no resource here
                 if (resource == null) return false
@@ -618,7 +635,7 @@ open class Tile : IsPartOfGameInfoSerialization {
 
     /** Shows important properties of this tile for debugging _only_, it helps to see what you're doing */
     override fun toString(): String {
-        val lineList = arrayListOf("TileInfo @$position")
+        val lineList = arrayListOf("Tile @$position")
         if (!this::baseTerrain.isInitialized) return lineList[0] + ", uninitialized"
         if (isCityCenter()) lineList += getCity()!!.name
         lineList += baseTerrain
@@ -812,6 +829,13 @@ open class Tile : IsPartOfGameInfoSerialization {
         val newUniqueMap = UniqueMap()
         for (terrain in allTerrains)
             newUniqueMap.addUniques(terrain.uniqueObjects)
+
+        lastTerrain = when {
+            terrainFeatures.isNotEmpty() -> ruleset.terrains[terrainFeatures.last()]
+                ?: getBaseTerrain()  // defense against rare edge cases involving baseTerrain Hill deprecation
+            naturalWonder != null -> getNaturalWonder()
+            else -> getBaseTerrain()
+        }
         terrainUniqueMap = newUniqueMap
     }
 
@@ -869,11 +893,10 @@ open class Tile : IsPartOfGameInfoSerialization {
                 removeRoad()
         } else {
             // otherwise use pillage/repair systems
-            if (canPillageTileImprovement()) {
+            if (canPillageTileImprovement())
                 improvementIsPillaged = true
-            } else {
+            else
                 roadIsPillaged = true
-            }
         }
     }
 
