@@ -20,7 +20,7 @@ class GameParameters : IsPartOfGameInfoSerialization { // Default values are the
     var maxNumberOfPlayers = 3
     var players = ArrayList<Player>().apply {
         add(Player(playerType = PlayerType.Human))
-        for (i in 1..3) add(Player())
+        repeat(3) { add(Player()) }
     }
     var randomNumberOfCityStates = false
     var minNumberOfCityStates = 6
